@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' package for R.
- * Copyright (C) 2013-2015, Marek Gagolewski and Bartek Tartanus
+ * Copyright (C) 2013-2016, Marek Gagolewski and Bartek Tartanus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -68,7 +68,10 @@
    "invalid UTF-8 byte sequence detected. perhaps you should try calling stri_enc_toutf8()"
 
 #define MSG__INVALID_ESCAPE \
-   "invalid escape sequence detected. Setting NA"
+   "invalid escape sequence detected. Setting NA"              \
+
+#define MSG__REPLACEMENT_ZERO \
+   "replacement has length zero"
 
 #define MSG__UNCONVERTABLE_CODE_POINT \
    "the Unicode codepoint \\U%08.8x cannot be converted to destination encoding"
@@ -165,7 +168,7 @@
    "string search failed"
 
 #define MSG__RESOURCE_ERROR_GET \
-   "required resource unavailable; try calling stri_install_check()"
+   "required ICU resource unavailable"
 
 #define MSG__RESOURCE_ERROR_APPLY \
    "error while applying operation"
